{/----------------------------------------------------------------------
//  Demoprogramm zur Klasse TFritzDataControl
//
//  Autor:  Christian "NineBerry" Schwarz
//  E-Mail: TFritzDataControl@NineBerry.de
//  Web:    http://www.NineBerry.de
//
//  Dieses Demoprogramm demonstriert die Verwendung der Klasse
//  TFritzDataControl zur Fernsteuerung des Programms Fritz!data
//  von AVM.
//
//  Das Programm verbindet sich mit dem AVM Data Call Center und
//  ldt von dort die Datei "FRISCR32.txt" aus dem Verzeichnis
//  "TOOLS\FDATASCR" auf den Desktop des aktuellen Computers.
//  Hinweis: Die Datei hat eine Gre von 800 kb. Das Laden dauert
//  etwa 2 Minuten.
//
//  History:
//    08/04/2000  Erste Version aufbauend auf Lounge-Testprogramm
//    13/04/2000  Danke an Heidi Hirschfeld vom AVM Support Team
//                fr die neue Version von F_OLE.DOC, in der die von
//                mir beschriebenen Fehler richtiggestellt sind und
//                einige weitere interessante Ergnzungen enthalten
//                sind.
//                Ich werde die neuen Informationen in den nchsten
//                Tagen integrieren
//    13/04/2000  Demo von GetCWDLocal und GetCWDRemote integriert
//    13/04/2000  Demo von GetDriveListLocal und GetDriveListRemote
//                integriert
//    13/04/2000  Demo von LFNSupported integriert
//    16/04/2000  Fehler bei nicht installiertem Fritz!data abgefangen
//    26/12/2000  Name der heruntergeladenen Datei gendert
//                Hinzufgen einer Anzeige der Dauer der Verbindung
//    08/09/2001  Verwenden von Application.HandleMessage zur Reduktion
//                der Prozessorauslastung
//                Verwendung eines Timers zur Aktualisierung der
//                Zeitanzeige
//
//
//  Die Klasse TFritzDataControl ist Freeware. Sie darf kostenlos und ohne
//  Einschrnkungen in (auch kommerziellen) Softwareprodukten verwendet
//  werden.
//  Nichtsdestotrotz behlt der Autor (Christian "NineBerry" Schwarz) alle
//  Rechte.
//
//  Die Weitergabe dieser Klasse ist nur in einer der folgenden Formen
//  zulssig:
//    x Als Teil eines kompilierten Anwendungsprogramms
//    x Als Teil eines Anwendungsprogrammes im Quelltext, nur in
//      unvernderter Form und inklusive dieser Textdatei.
//    x In unvernderter Form, inklusive dieser Textdatei unentgeldlich
//    x In unvernderter Form, inklusive dieser Textdatei, gegen eine Gebhr
//      (z.B. auf CD-Beilagen von Zeitschriften oder hnliches)
//      nur nach Rcksprache mit dem Autor (Christian "NineBerry" Schwarz)
//
//  Falls du diese Klasse zur Erstellung von Anwendungen verwendest,
//  wrde ich mich sehr freuen ber:
//    x Namentliche Nennung in der Dokumentation deiner Anwendung.
//    x Eine nette E-Mail oder Postkarte mit Informationen ber dich
//
//----------------------------------------------------------------------}
unit dlgMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Buttons, FritzDataControl, Shlobj, ExtCtrls;

const
  PhoneNumber = '030 39984300';  // AVM Data Call Center (Berlin)
  TargetDir = 'TOOLS\FDATASCR';  // Verzeichnis im AVM DCC
  TargetFile = 'FRISCR32.EXE';      // Dateiname im AVM DCC

type
  TfrmMain = class(TForm)
    redLog: TRichEdit;
    btnStart: TBitBtn;
    btnAbort: TBitBtn;
    lblTime: TLabel;
    Timer: TTimer;
    procedure btnStartClick(Sender: TObject);
    procedure btnAbortClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TimerTimer(Sender: TObject);
  private
    fControl: TFritzDataControl;
    fAborted: Boolean;
    fStartTime: TDateTime;
    procedure PrepareLog;
    procedure LogScrollInView;
    procedure AddLogNewLine;
    procedure AddLogStatus(S: String);
    procedure AddLogError(S: String);
    procedure AddLogStartDirListing;
    procedure AddLogEndDirListing;
    procedure AddLogDirListingItem(Item: TFritzDataFileInfo);

    procedure WaitUntilReady;
    procedure CheckForErrors;
    procedure PrintDirectoryOutput;
    procedure Action;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}


{/----------------------------------------------------------------------
//  Dient zur Bestimmung des Desktop-Ordners
//----------------------------------------------------------------------}
function GetSpecialFolderLocation(Kind: Integer): string;
var
  ItemID: PItemIDList;
  S: string;
  EndPos: Integer;
begin
  Result:= EmptyStr;
  if SHGetSpecialFolderLocation(0, Kind, ItemID) = NOERROR
  then begin
    SetLength(S, MAX_PATH);
    if SHGetPathFromIDList(ItemID, PChar(S)) then
    begin
      EndPos:= Pos(#0, S);
      Result:= Copy(S, 1, EndPos-1);
    end;
  end;
end;

{/----------------------------------------------------------------------
//  Text mit genau Size Zeichen zurckgeben, dabei evt. rechts
//  mit leerzeichen auffllen
//----------------------------------------------------------------------}
function PadLeft(Text: String; Size: Integer): String;
begin
  Result:= Copy(Text, 1, Size);
  while Length(Result) < Size do
    Result:= Result + ' ';
end;

{/----------------------------------------------------------------------
//  Text mit genau Size Zeichen zurckgeben, dabei evt. links
//  mit leerzeichen auffllen
//----------------------------------------------------------------------}
function PadRight(Text: String; Size: Integer): String;
begin
  Result:= Copy(Text, 1, Size);
  while Length(Result) < Size do
    Result:= ' ' + Result;
end;

{/----------------------------------------------------------------------
//  Cursor ans Ende des Logs setzen, so dass neuer Text dort
//  eingefgt wird
//----------------------------------------------------------------------}
procedure TfrmMain.PrepareLog;
begin
  // Cursor an letzte Position
  redLog.SelStart:= Length(redLog.Lines.Text);
  redLog.SelLength:= 0;
end;

{/----------------------------------------------------------------------
//  Log Scrollen
//----------------------------------------------------------------------}
procedure TfrmMain.LogScrollInView;
begin
  redLog.Perform(EM_ScrollCaret, 0, 0);
end;

{/----------------------------------------------------------------------
//  Leere Zeile im Log einfgen
//----------------------------------------------------------------------}
procedure TfrmMain.AddLogNewLine;
begin
  PrepareLog;
  redLog.Lines.Add(EmptyStr);
  LogScrollInView;
end;

{/----------------------------------------------------------------------
//  Statusmeldung im Log ausgeben
//----------------------------------------------------------------------}
procedure TfrmMain.AddLogStatus(S: String);
begin
  PrepareLog;

  redLog.SelAttributes.Style:= [];
  redLog.Lines.Add('>  ' + S);

  LogScrollInView;
end;

{/----------------------------------------------------------------------
//  Fehler im Log ausgeben
//----------------------------------------------------------------------}
procedure TfrmMain.AddLogError(S: String);
begin
  PrepareLog;

  redLog.SelAttributes.Style:= [fsbold];
  redLog.Lines.Add('!! ' + S);

  LogScrollInView;
end;

{/----------------------------------------------------------------------
//  Anfang einer Verzeichnisliste
//----------------------------------------------------------------------}
procedure TfrmMain.AddLogStartDirListing;
begin
  PrepareLog;

  redLog.SelAttributes.Name:= 'courier';
  AddLogNewLine;
  redLog.Lines.Add('------------------------------------------------------------');

  LogScrollInView;
end;

{/----------------------------------------------------------------------
//  Ende einer Verzeichnisliste
//----------------------------------------------------------------------}
procedure TfrmMain.AddLogEndDirListing;
begin
  PrepareLog;

  redLog.Lines.Add('------------------------------------------------------------');
  AddLogNewLine;
  redLog.SelAttributes.Name:= 'MS Sans Serif';

  LogScrollInView;
end;

{/----------------------------------------------------------------------
//  Verzeichniseinntrag im Log anzeigen
//----------------------------------------------------------------------}
procedure TfrmMain.AddLogDirListingItem(Item: TFritzDataFileInfo);
var
  S: String;
begin
  PrepareLog;

  S:= PadLeft(Item.FileName, 28);

  if Item.Typ = fdftDirectory then
    S:= S + ' ' + PadRight('<DIR>', 7)
  else
    S:= S + ' ' + PadRight(IntToStr(Item.Size), 7);

  S:= S + ' ' + PadLeft(DateTimeToStr(Item.LastChange), 20);

  redLog.Lines.Add(S);

  LogScrollInView;
end;


{/----------------------------------------------------------------------
//  Wartet, bis eine asynchrone Aktion beendet ist
//  Reagiert auf Abbruchsanforderung
//----------------------------------------------------------------------}
procedure TfrmMain.WaitUntilReady;
var
  ConnTime: TDateTime;
begin
  repeat
    ConnTime:= Now - fStartTime;
    lblTime.Caption:= FormatDateTime('HH:MM:SS', ConnTime);
      
    Application.HandleMessage;
    if fAborted then
    begin
      AddLogError('Abbruch');
      fControl.Abort;
      repeat
        Application.HandleMessage;
      until not fControl.Busy;
      Abort;
    end;
  until not fControl.Busy;
end;

{/----------------------------------------------------------------------
//  Gibt Verzeichnislistung aus
//----------------------------------------------------------------------}
procedure TfrmMain.PrintDirectoryOutput;
var
  FileInfo: TFritzDataFileInfo;
begin
  AddLogStartDirListing;
  while fControl.GetNextDirItem(FileInfo) <> fdftEOL do
  begin
    AddLogDirListingItem(FileInfo);
  end;
  AddLogEndDirListing;
end;

{/----------------------------------------------------------------------
//  berprft, ob bei letzter Aktion ein Fehler aufgetreten ist
//  Falls ja, wird der Fehlertext angezeigt und eine stille Exception
//  ausgelst, um weitere Aktionen zu unterbinden 
//----------------------------------------------------------------------}
procedure TfrmMain.CheckForErrors;
begin
  if fControl.ErrorOccured then
  begin
    if fControl.ErrorMessage = EmptyStr then
      AddLogError('Unbekannter Fehler aufgetreten.')
    else
      AddLogError(fControl.ErrorMessage);
    Abort;
  end;
end;

{/----------------------------------------------------------------------
//  Fhrt die eigentlichen Aktionen nacheinander aus.
//----------------------------------------------------------------------}
procedure TfrmMain.Action;
var
  DesktopDir: String;
  Info: String;
  BInfo: Boolean;
begin

  fStartTime:= Now;

  // Fritz!data darf noch nicht laufen
  if TFritzDataControl.FritzDataAlreadyRuns then
  begin
    MessageDlg('Fritz!data luft schon.', mtError, [mbOk], 0);
    Abort;
  end;

  fAborted:= False;

  try
    fControl:= TFritzDataControl.Create;
  except
    MessageDlg('Fritz!data ab Version 2.02 Revision 1.26 ist auf diesem PC nicht (korrekt) installiert.',
      mtError, [mbOk], 0);
    Abort;
  end;
  try
    // Desktop-Verzeichnis ermitteln
    DesktopDir:= GetSpecialFolderLocation(CSIDL_DESKTOPDIRECTORY);

    // Verfgbare Laufwerke lokal
    AddLogStatus('Ermittle lokal verfgbare Laufwerke...');
    Info:= fControl.GetDriveListLocal;
    CheckForErrors;
    AddLogStatus(Format('Verfgbare Laufwerke: %s', [Info]));

    // lokal in Desktop-Verzeichnis wechseln
    AddLogStatus(Format('Wechsle lokal in Verzeichnis %s...', [DesktopDir]));
    fControl.ChDirLocal(DesktopDir);
    WaitUntilReady;
    CheckForErrors;

    // Aktuelles lokales Verzeichnis anzeigen
    AddLogStatus('Ermittle aktuelles lokales Verzeichnis');
    Info:= fControl.GetCWDLocal;
    CheckForErrors;
    AddLogStatus(Format('Aktuelles lokales Verzeichnis ist %s', [Info]));
    
    // Verzeichnisinhalt Desktop
    AddLogStatus(Format('Lese Verzeichnisinhalt von %s...', [DesktopDir]));
    fControl.DirLocal;
    WaitUntilReady;
    CheckForErrors;

    PrintDirectoryOutput;

    // Verbindung zu AVM Data Call Center aufbauen
    AddLogStatus(Format('Whle Nr. %s...', [PhoneNumber]));
    fControl.Connect(PhoneNumber, '', '', [fdcIDTrans, fdcIDTransCompression]);
    WaitUntilReady;
    CheckForErrors;

    // Servernamen anzeigen
    AddLogStatus(Format('Verbunden mit %s', [fControl.GetServerName]));

    // Verfgbare Laufwerke auf Server
    AddLogStatus('Ermittle verfgbare Laufwerke auf Server...');
    Info:= fControl.GetDriveListRemote;
    CheckForErrors;
    AddLogStatus(Format('Verfgbare Laufwerke: %s', [Info]));

    // Lange Dateinamen untersttzt?
    AddLogStatus('Ermittle Verfgbarkeit von langen Dateinamen...');
    BInfo:= fControl.LFNSupported;
    CheckForErrors;
    if BInfo then
      AddLogStatus('Lange Dateinamen werden untersttzt.')
    else
      AddLogStatus('Lange Dateinamen werden nicht untersttzt.');

    // Aktuelles Verzeichnis auf Server anzeigen
    AddLogStatus('Ermittle aktuelles Verzeichnis auf Server');
    Info:= fControl.GetCWDRemote;
    CheckForErrors;
    AddLogStatus(Format('Aktuelles Verzeichnis auf Server ist %s', [Info]));

    // Verzeichnisinhalt Server Startverzeichnis
    AddLogStatus('Lese Verzeichnisinhalt auf Server...');
    fControl.DirRemote;
    WaitUntilReady;
    CheckForErrors;

    PrintDirectoryOutput;

    // Verzeichnis auf Server wechseln
    AddLogStatus(Format('Wechsle in Verzeichnis %s...', [TargetDir]));
    fControl.ChDirRemote(TargetDir);
    WaitUntilReady;
    CheckForErrors;

    // Aktuelles Verzeichnis auf Server anzeigen
    AddLogStatus('Ermittle aktuelles Verzeichnis auf Server');
    Info:= fControl.GetCWDRemote;
    CheckForErrors;
    AddLogStatus(Format('Aktuelles Verzeichnis auf Server ist %s', [Info]));

    // Verzeichnisinhalt Verzeichnis auf Server
    AddLogStatus(Format('Lese Verzeichnisinhalt von %s...', [TargetDir]));
    fControl.DirRemote;
    WaitUntilReady;
    CheckForErrors;

    PrintDirectoryOutput;

    // Datei vom Server auf den Desktop laden
    AddLogStatus(Format('Downloade Datei %s...', [TargetFile]));
    fControl.Download(TargetFile, False);
    WaitUntilReady;
    CheckForErrors;

    // Verbindung beenden
    AddLogStatus('Beende Verbindung...');
    fControl.Disconnect;
    WaitUntilReady;
    CheckForErrors;

    AddLogStatus('Fertig!');
    AddLogNewLine;
    AddLogNewLine;
  finally
    fControl.Free;
  end;
end;

{/----------------------------------------------------------------------
//  Button "Start"
//  Aktion starten
//----------------------------------------------------------------------}
procedure TfrmMain.btnStartClick(Sender: TObject);
begin
  btnStart.Enabled:= False;
  btnAbort.Enabled:= True;
  try
    Action;
  finally
    btnStart.Enabled:= True;
    btnAbort.Enabled:= False;
  end;
end;

{/----------------------------------------------------------------------
//  Button "Abbrechen"
//  Gewnschten Abbruch signalisieren
//----------------------------------------------------------------------}
procedure TfrmMain.btnAbortClick(Sender: TObject);
begin
  fAborted:= True;
end;

{/----------------------------------------------------------------------
//  Beim Schlieen des Fensters Abbruch signalisieren
//----------------------------------------------------------------------}
procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if btnAbort.Enabled
    then btnAbort.Click;
end;

procedure TfrmMain.TimerTimer(Sender: TObject);
begin
  //
end;

end.
