{/----------------------------------------------------------------------
//  TFritzDataControl
//
//  Klasse, die die Fernsteuerung von AVM Fritz!Data ab Version
//  2.0 kapselt.
//
//  Autor:  Christian "NineBerry" Schwarz
//  E-Mail: TFritzDataControl@NineBerry.de
//  Web:    http://www.NineBerry.de
//
//  History:
//    22/03/2000  TFritzDataMessageHandler und IFritzDataMessageHandler
//                Grundlagen implementiert und getestet.
//    22/03/2000  Klasse TFritzDataControl eingefhrt und mit
//                TFritzDataMessageHandler verknpft.
//    22/03/2000  Grundlegende Funktionalitt Busy/Ready/PutMessage
//    22/03/2000  Connect und Disconnect implementiert
//    23/03/2000  DirRemote und notwendige Erweiterungen implementiert
//                (Typen, fDirectories, fFiles, nderungen an PutMessage,
//                GetNextDirItem)
//    24/03/2000  LocalReady und CheckLocalReady hinzugefgt fr Aktionen,
//                die keine Serververbindungen bentigen
//    24/03/2000  DirLocal hinzugefgt
//    24/03/2000  ChDirLocal hinzugefgt
//    24/03/2000  Verhalten von fBusy korrigiert. Vor Aktion auf True
//                setzen, Bei Ende-Nachricht oder Fehler auf False setzen
//    24/03/2000  GetCWDLocal hinzugefgt -> nicht zum Laufen gebracht
//                -> Fehler in f_ole.doc ???
//    24/03/2000  MkDirLocal hinzugefgt
//    24/03/2000  DeleteLocal hinzugefgt
//    24/03/2000  GetServerName hinzugefgt
//    24/03/2000  Aus HandlePResult eine Funktion gemacht
//    25/03/2000  ChDirRemote hinzugefgt
//    26/03/2000  MkDirRemote hinzugefgt
//    26/03/2000  DeleteRemote hinzugefgt
//    26/03/2000  Upload hinzugefgt
//    26/03/2000  Download hinzugefgt
//    26/03/2000  LFNSupported hinzugefgt-> nicht zum Laufen gebracht
//                -> Fehler in f_ole.doc ???
//    26/03/2000  GetConnStatus hinzugefgt
//    07/04/2000  StringReplace durch ClearDots ersetzt -> Komp. mit D3
//    07/04/2000  GetCWDRemote hinzugefgt -> nicht zum Laufen gebracht
//                -> Fehler in f_ole.doc ???
//    07/04/2000  Abort hinzugefgt
//    07/04/2000  GetDriveListLocal hinzugefgt -> nicht zum Laufen gebracht
//                -> Fehler in f_ole.doc ???
//    07/04/2000  GetDriveListRemote hinzugefgt -> nicht zum Laufen gebracht
//                -> Fehler in f_ole.doc ???
//    07/04/2000  GetCAPIISDNErrorMessage hinzugefgt -> nicht zum Laufen gebracht
//                -> Fehler in f_ole.doc ???
//    08/04/2000  Busy nicht nur von internem Flag abhngig, sondern
//                auch von tatschlichem Serverstatus -> Methode GetBusy
//    08/04/2000  Erweiterung von TFritzDataFileInfo um Feld Typ
//    08/04/2000  Definition des Interfaces IFritzDataMessageHandler aus
//                der von Delphi erzeugten in diese Unit verschoben
//                und auf das notwendigste verkrzt.
//    09/04/2000  Korrektur des Rechtschreibfehlers Adress -> Address
//                in Connect.
//    10/04/2000  Test unter Delphi 3. Dabei ergaben sich einige kleinere
//                syntaktische nderungen, sowie die Notwendigkeit, die
//                *.TLB - Datei manuell als Resource einzubinden
//    13/04/2000  Danke an Heidi Hirschfeld vom AVM Support Team
//                fr die neue Version von F_OLE.DOC, in der die von
//                mir beschriebenen Fehler richtiggestellt sind und
//                einige weitere interessante Ergnzungen enthalten
//                sind.
//                Ich werde die neuen Informationen in den nchsten
//                Tagen integrieren
//    13/04/2000  GetCWDLocal und GetCWDRemote korrigiert und getestet
//                -> funktionieren nun zufriedenstellend
//    13/04/2000  GetDriveListLocal und GetDriveListRemote korrigiert
//                und getestet -> funktionieren nun zufriedenstellend
//    13/04/2000  LFNSupported korrigiert und getestet
//                -> funktioniert nun zufriedenstellend
//    13/04/2000  GetCAPIISDNErrorMessage korrigiert und getestet
//                -> funktioniert nun zufriedenstellend
//    13/04/2000  ErrorHandling in HandlePResult an Beschreibung
//                in neuer f_ole.doc angepasst.
//    13/04/2000  Verfgbarkeit von WildCards dokumentiert
//    26/12/2000  Funktionieren auch mit Version 3.0 von
//                Fritz!data dokumentiert
//    06/09/2001  Von TAutoIntfObject statt von TAutoObject erben.
//                Damit muss die Co-Klasse nicht mehr im System
//                registriert werden.
//    06/09/2001  Kompatibilitt mit Delphi 6 (Uses-Klausel)
//    06/09/2001  Wenn Symbol "USEALIENTYPELIB" gesetzt ist, wird
//                die TypeLibrary mit der Definition von
//                IFritzDataMessageHandler nicht mehr eingebunden.
//                Es wird dann davon ausgegangen, dass dies an
//                anderer Stelle passiert.
//
//  Die Klasse TFritzDataControl ist Freeware. Sie darf kostenlos und ohne
//  Einschrnkungen in (auch kommerziellen) Softwareprodukten verwendet
//  werden.
//  Nichtsdestotrotz behlt der Autor (Christian "NineBerry" Schwarz) alle
//  Rechte.
//
//  Die Weitergabe dieser Klasse ist nur in einer der folgenden Formen
//  zulssig:
//    x Als Teil eines kompilierten Anwendungsprogramms
//    x Als Teil eines Anwendungsprogrammes im Quelltext, nur in
//      unvernderter Form und inklusive dieser Textdatei.
//    x In unvernderter Form, inklusive dieser Textdatei unentgeldlich
//    x In unvernderter Form, inklusive dieser Textdatei, gegen eine Gebhr
//      (z.B. auf CD-Beilagen von Zeitschriften oder hnliches)
//      nur nach Rcksprache mit dem Autor (Christian "NineBerry" Schwarz)
//
//  Falls du diese Klasse zur Erstellung von Anwendungen verwendest,
//  wrde ich mich sehr freuen ber:
//    x Namentliche Nennung in der Dokumentation deiner Anwendung.
//    x Eine nette E-Mail oder Postkarte mit Informationen ber dich
//
//----------------------------------------------------------------------}
unit FritzDataControl;

interface
uses
  {$IFDEF VER140}
  Variants,
  {$ENDIF}
  Classes,
  Windows,
  Dialogs,
  SysUtils,
  ComObj,
  ComServ;

{/----------------------------------------------------------------------
//  Definition der Schnittstelle IFritzDataMessageHandler
//  Aus von Delphi erzeugter Unit in diese bernommen
//----------------------------------------------------------------------}
const
  CLASS_FritzDataMessageHandler: TGUID = '{548F9EC8-00EB-11D4-A344-525400EA3749}';

type
  IFritzDataMessageHandler = interface(IDispatch)
    ['{548F9EC6-00EB-11D4-A344-525400EA3749}']
    procedure PutMessage(Message: Smallint; LParam: Integer; VParam: OleVariant;
                         var pResult: Integer); safecall;
  end;

  {/----------------------------------------------------------------------
  //  Verbindungsoptionen, bentigt fr Verbindungsaufbau, sowie fr
  //  Abfrage der Verbindungsoptionen
  //----------------------------------------------------------------------}
type
  TFritzDataConnectOption = (fdcIDTrans, fdcTwoChannel, fdcIDTransCompression,
    fdcV42Compression);
  TFritzDataConnectOptions = set of TFritzDataConnectOption;

  {/----------------------------------------------------------------------
  //  Typen im Zusammenhang mit dem Lesen eines Verzeichnisses
  //----------------------------------------------------------------------}
  TFritzDataFileType = (fdftDirectory, fdftFile, fdftEOL);
  TFritzDataFileInfo = record
    Typ: TFritzDataFileType;
    FileName: TFileName;
    Size: Integer;
    LastChange: TDateTime;
  end;

  {/----------------------------------------------------------------------
  //  Klasse TFritzDataMessageHandler ist ein Automatisierungsobjekt und
  //  dient als Callback fr Fritz!Data.
  //  Fritz!Data gibt Informationen zurck, indem es die Methode
  //  PutMessage der Klasse aufruft.
  //----------------------------------------------------------------------}
  // Forward-Deklaration der Klasse TFritzDataControl
  TFritzDataControl = class;

  TFritzDataMessageHandler = class(TAutoIntfObject, IFritzDataMessageHandler)
  private
    fGiveMessagesTo: TFritzDataControl;
    procedure PutMessage(Message: Smallint; LParam: Integer;
      VParam: OleVariant; var pResult: Integer); safecall;
  public
    constructor Create; 
  end;

  {/----------------------------------------------------------------------
  //  Klasse TFritzDataControl stellt die eigentliche Schnittstelle
  //  zum Programm dar. Sie kapselt eine COM-Verbindung zu Fritz!Data
  //  und ermglicht dadurch die Fernsteuerung dieses Programms.
  //  Es kann immer nur eine Instanz von TFritzDataControl gleichzeitig
  //  geben, und das auch nur dann, wenn Fritz!Data nicht schon luft.
  //  Eine Instanz sollte nur bei Bedarf erzeugt und so frh wie mglich
  //  wieder freigegeben werden.
  //----------------------------------------------------------------------}
  TFritzDataControl = class(TObject)
  private
    fBusy: Boolean;
    fConnected: Boolean;
    fErrorOccured: Boolean;
    fErrorNr: Integer;
    fErrorMessage: string;
    fFritzData: OleVariant;
    fMessageHandler: TFritzDataMessageHandler;
    fDirectories: TStringList;
    fFiles: TStringList;
    function GetReady: Boolean;
    function GetBusy: Boolean;
    function GetLocalReady: Boolean;
    procedure ClearErrorState;
    function CheckReady: Boolean;
    function CheckLocalReady: Boolean;
    function HandlePResult(PResult: Integer): Boolean;
  public
    constructor Create;
    destructor Destroy; override;

    class function FritzDataAlreadyRuns: Boolean;

    // Folgende Methoden laufen asynchron

    procedure Connect(Address, UserName, Password: string; Options:
      TFritzDataConnectOptions);
    procedure Disconnect;

    procedure DirRemote;
    procedure DirLocal;

    procedure ChDirLocal(NewPath: string);
    procedure ChDirRemote(NewPath: string);

    procedure MkDirLocal(DirName: string);
    procedure MkDirRemote(DirName: string);

    procedure DeleteLocal(FileName: string);
    procedure DeleteRemote(FileName: string);

    procedure Upload(FileName: string; Move: Boolean);
    procedure Download(FileName: string; Move: Boolean);

    procedure Abort;

    // Folgende Methoden/Properties laufen nicht asynchron
    function GetNextDirItem(var FileInfo: TFritzDataFileInfo):
      TFritzDataFileType;
    function GetCWDLocal: string;
    function GetCWDRemote: string;
    function GetDriveListLocal: String;
    function GetDriveListRemote: String;
    function GetServerName: string;
    function LFNSupported: Boolean;
    function GetConnStatus: TFritzDataConnectOptions;
    function GetCAPIISDNErrorMessage(Error: Word): String;

    property Ready: Boolean read GetReady;
    property LocalReady: Boolean read GetLocalReady;
    property Busy: Boolean read GetBusy;
    property ErrorOccured: Boolean read fErrorOccured;
    property ErrorNr: Integer read fErrorNr;
    property ErrorMessage: string read fErrorMessage;
  end;

const
  {/----------------------------------------------------------------------
  //  Konstanten fr Fehlermeldungen (eigene und von Fritz! definiert)
  //----------------------------------------------------------------------}
  // Eigene
  FDERR_UNKNOWN = $FFFF;
  FDERR_WRONGPARAMETERS = $FFFE;
  FDERR_NOTPOSSIBLE = $FFFD;
  FDERR_NOTREADY = $FFFC;

  // Fritz!Data
  FDERR_GENERALFAILURE = $40000;
  FDERR_FILETRANSFER = $40001;
  FDERR_FILEACCESS = $4001E;
  FDERR_PATHNOTFOUND = $4001F;
  FDERR_ACCESSDENIED = $40020;
  FDERR_FILEOPEN = $40021;
  FDERR_TIMEOUT = $40022;
  FDERR_LOGINFAILURE = $40023;
  FDERR_FILEALREADYEXISTS = $40026;
  FDERR_IDLETIMEOUT = $40027;
  FDERR_DOSERROR = $40028;
  FDERR_NOREPLY = $40064;
  FDERR_NODISKSPACE = $40065;
  FDERR_DISKERROR = $40066;
  FDERR_DATAEXPECTED = $40067;
  FDERR_NOACK = $40068;
  FDERR_INVALIDRECIPIENT = $40069;
  FDERR_OUTOFMEMORY = $4006A;
  FDERR_PATHACCESS = $4006C;
  FDERR_FILENOTFOUND = $4006D;
  FDERR_CONNCOLLISION = $4006E;
  FDERR_ABORT = $4006F;
  FDERR_INCOMPRECIPIENT = $40070;

resourcestring
  {/----------------------------------------------------------------------
  //  Texte fr Fehlermeldungen
  //----------------------------------------------------------------------}
  // Eigene
  FDERRS_UNKNOWN = 'Unbekannter Fehler';
  FDERRS_WRONGPARAMETERS = 'Falsche Parameter angegeben';
  FDERRS_NOTPOSSIBLE = 'Aktion nicht durchfhrbar';
  FDERRS_NOTREADY = 'FRITZ!Data noch beschftigt oder keine Verbindung';

  // Fritz!Data
  FDERRS_GENERALFAILURE = 'Nicht weiter spezifizierter Fritz!-Fehler aufgetreten';
  FDERRS_FILETRANSFER = 'Fehler beim Dateitransfer';
  FDERRS_FILEACCESS = 'DOS-Zugriff verweigert';
  FDERRS_PATHNOTFOUND = 'Datei/Ordner nicht gefunden';
  FDERRS_ACCESSDENIED = 'Zugriff auf Datei/Ordner verweigert';
  FDERRS_FILEOPEN = 'Fehler beim ffnen der Datei';
  FDERRS_TIMEOUT = 'Abbruch wegen Zeitberschreitung';
  FDERRS_LOGINFAILURE = 'Fehlerhafte Anmeldung beim Server';
  FDERRS_FILEALREADYEXISTS = 'Datei/Ordner existiert bereits';
  FDERRS_IDLETIMEOUT = 'Abbruch wegen Benutzer-Zeitberschreitung';
  FDERRS_DOSERROR = 'DOS-Fehler';
  FDERRS_NOREPLY = 'Gegenstelle antwortet nicht';
  FDERRS_NODISKSPACE = 'Nicht gengend Diskettenplatz';
  FDERRS_DISKERROR = 'Diskettenfehler';
  FDERRS_DATAEXPECTED = 'Gegenstelle erwartet Daten';
  FDERRS_NOACK = 'Gegenstelle besttigt nicht';
  FDERRS_INVALIDRECIPIENT = 'Gegenstelle ist ungltig';
  FDERRS_OUTOFMEMORY = 'Nicht gengend Speicher';
  FDERRS_PATHACCESS = 'Pfad nicht zugnglich';
  FDERRS_FILENOTFOUND = 'Datei nicht gefunden';
  FDERRS_CONNCOLLISION = 'Verbindungskollision';
  FDERRS_ABORT = 'Funktion abgebrochen';
  FDERRS_INCOMPRECIPIENT = 'Gegenstelle ist nicht kompatibel';

implementation

{$IFNDEF USEALIENTYPELIB}
// Res-Datei mit Typelibrary linken
{$R FritzDataControl.RES}
{$ENDIF}

const
  {/----------------------------------------------------------------------
  //  Name des Fensters von Fritz!Data. Anahnd dieses Namens ermittelt
  //  FritzDataAlreadyRuns, ob Fritz!Data schon luft.
  //  Hinweis: Dieser Name wird auch von Fritz!data Version 3 verwendet
  //----------------------------------------------------------------------}
  FritzDataWindowClassName = 'AVM_FRITZ!data 2.0';

{/----------------------------------------------------------------------
//  Methode TFritzDataMessageHandler.PutMessage
//  Empfngt Nachrichten von Fritz!Data und leitet sie an die aktuelle
//  Instanz von TFritzDataConrol weiter.
//----------------------------------------------------------------------}
procedure TFritzDataMessageHandler.PutMessage(Message: Smallint;
  LParam: Integer; VParam: OleVariant; var pResult: Integer);

  {/----------------------------------------------------------------------
  //  Im Fehlerfall Status, Nr und Text an TFritzDataControl-Objekt
  //  weiterreichen
  //----------------------------------------------------------------------}
  function ErrorsOccured: Boolean;
  begin
    Result:= LParam <> 0;
    if Result then
    begin
      fGiveMessagesTo.fErrorOccured:= true;
      fGiveMessagesTo.fErrorNr:= LParam;
      fGiveMessagesTo.fErrorMessage:= VParam;
    end;
  end;

begin
  // TFritzDataControl-Objekt muss existieren
  if not Assigned(fGiveMessagesTo) then
    Exit;

  case Message of
    1: // Dateilisting (Verzeichnis)
      begin
        fGiveMessagesTo.fDirectories.Add(VParam);
      end;
    2: // Dateilisting (Datei)
      begin
        fGiveMessagesTo.fFiles.Add(VParam);
      end;
    10: // Dateilisting (Ende)
      begin
        fGiveMessagesTo.fBusy:= False;
      end;
    3: // Verbindungsaufbau
      begin
        fGiveMessagesTo.fBusy:= False;
        if not ErrorsOccured then
          fGiveMessagesTo.fConnected:= True;
      end;
    4: // Verbindungsabbau
      begin
        fGiveMessagesTo.fBusy:= False;
        fGiveMessagesTo.fConnected:= False;
      end;
    5: // Verzeichniswechsel
      begin
        fGiveMessagesTo.fBusy:= False;
        ErrorsOccured;
      end;
    6: // Verzeichnis erstellen
      begin
        fGiveMessagesTo.fBusy:= False;
        ErrorsOccured;
      end;
    7: // Kopieren
      begin
        fGiveMessagesTo.fBusy:= False;
        ErrorsOccured;
      end;
    8: // Verschieben
      begin
        fGiveMessagesTo.fBusy:= False;
        ErrorsOccured;
      end;
    9: // Lschen
      begin
        fGiveMessagesTo.fBusy:= False;
        ErrorsOccured;
      end;
    $107, $108: // Frage: berschreiben von Dateien
      begin
        pResult:= 4; // Alle berschreiben
      end;
  end;
end;

{/----------------------------------------------------------------------
//  Konstruktor TFritzDataMessageHandler.Create
//----------------------------------------------------------------------}
constructor TFritzDataMessageHandler.Create;
begin
  inherited Create(ComServer.TypeLib, IFritzDataMessageHandler);

  fGiveMessagesTo:= nil;
end;

{/----------------------------------------------------------------------
//  Konstruktor TFritzDataControl.Create
//  Bricht ab, falls Fritz!Data schon luft.
//  Erzeugt COM-Verbindung zu Fritz!Data
//  Initialisert alle Attribute der Klasse
//----------------------------------------------------------------------}
constructor TFritzDataControl.Create;
begin
  inherited Create;

  if FritzDataAlreadyRuns then
    Abort;

  fFritzData:= CreateOleObject('FritzData.Document');
  fMessageHandler:= TFritzDataMessageHandler.Create;
  fMessageHandler.fGiveMessagesTo:= Self;
  fFritzData.SetClientId(fMessageHandler as IFritzDataMessageHandler);

  fBusy:= False;
  fConnected:= False;
  ClearErrorState;

  fDirectories:= TStringList.Create;
  fFiles:= TStringList.Create;
end;

{/----------------------------------------------------------------------
//  Destruktor TFritzDataControl.Destroy
//  Gibt Objekt-Felder frei
//  Lst COM-Verbindung zu Fritz-Data
//----------------------------------------------------------------------}
destructor TFritzDataControl.Destroy;
begin
  fDirectories.Free;
  fFiles.Free;

  fFritzData:= null;
  fMessageHandler:= nil;
  inherited;
end;

{/----------------------------------------------------------------------
//  Klassenmethode TFritzDataControl.FritzDataAlreadyRuns
//  Gibt True zurck, wenn Fritz!Data gerade luft, ansonsten False
//----------------------------------------------------------------------}
class function TFritzDataControl.FritzDataAlreadyRuns: Boolean;
begin
  Result:= FindWindow(FritzDataWindowClassName, nil) <> 0;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.GetReady
//  Gibt True zurck, wenn Fritz!Data fr weitere Befehle bereit ist.
//----------------------------------------------------------------------}
function TFritzDataControl.GetReady: Boolean;
begin
  Result:= fConnected and not Busy;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.GetBusy
//  Gibt True zurck, wenn Fritz!data gerade beschftigt ist
//----------------------------------------------------------------------}
function TFritzDataControl.GetBusy: Boolean;
  function IsBusyConnection: Boolean;
  var
    PInfo: OleVariant;
    PResult: Integer;
    Flag: SmallInt;
    Info: Word;
  begin
    Flag:= 256; // Get ConnInfo
    PInfo:= Unassigned;
    fFritzData.GetInfo(Flag, PInfo, PResult);
    Info:= PInfo;
    Result:= (Info and 5) <> 0;
  end;
begin
  Result:= fBusy or IsBusyConnection;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.GetLocalReady
//  Gibt True zurck, wenn Fritz!Data fr weitere lokale Befehle
//  bereit ist.
//----------------------------------------------------------------------}
function TFritzDataControl.GetLocalReady: Boolean;
begin
  Result:= not fBusy;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.ClearErrorState
//  Setzt Fehlerstatus zurck.
//----------------------------------------------------------------------}
procedure TFritzDataControl.ClearErrorState;
begin
  fErrorOccured:= False;
  fErrorNr:= 0;
  fErrorMessage:= EmptyStr;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.CheckReady
//  Gibt True zurck, wenn Fritz!Data fr weitere Befehle bereit ist.
//  Wenn nein, wird ein entsprechender Fehlerstatus gesetzt.
//----------------------------------------------------------------------}
function TFritzDataControl.CheckReady: Boolean;
begin
  Result:= Ready;
  if not Result then
  begin
    fErrorOccured:= True;
    fErrorNr:= FDERR_NOTREADY;
    fErrorMessage:= FDERRS_NOTREADY;
  end;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.CheckLocalReady
//  Gibt True zurck, wenn Fritz!Data fr weitere lokale Befehle
//  bereit ist.
//  Wenn nein, wird ein entsprechender Fehlerstatus gesetzt.
//----------------------------------------------------------------------}
function TFritzDataControl.CheckLocalReady: Boolean;
begin
  Result:= LocalReady;
  if not Result then
  begin
    fErrorOccured:= True;
    fErrorNr:= FDERR_NOTREADY;
    fErrorMessage:= FDERRS_NOTREADY;
  end;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.HandlePResult
//  Wertet den Rckgabewert eines Aufrufs einer Funktion von Fritz!data
//  aus. Versetzt dann das Objekt in den entsprechenden Zustand
//  (Busy, Fehlerfall, Verbindung ja/nein)
//  Gibt Im Fehlerfall False zurck, bei Erfolgsmeldung True
//----------------------------------------------------------------------}
function TFritzDataControl.HandlePResult(PResult: Integer): Boolean;
var
  HiWord, LoWord: Word;
begin
  HiWord:= PResult div $10000;
  LoWord:= PResult mod $10000; 

  case HiWord of
    0, 1:
    begin
      // Erfolgreich, bei 1 muss evt. auf Nachricht
      // gewartet werden
      Result:= True;
    end;
    2:
    begin
      // Parameter ungltig
      fErrorOccured:= True;
      fErrorNr:= FDERR_WRONGPARAMETERS;
      fErrorMessage:= FDERRS_WRONGPARAMETERS;
      fBusy:= False;
      Result:= False;
    end;
    3:
    begin
      // CAPI- oder ISDN-Fehler
      fErrorOccured:= True;
      fBusy:= False;
      Result:= False;
      fErrorMessage:= GetCAPIISDNErrorMessage(LoWord);
      fErrorNr:= LoWord;
    end;
    4:
    begin
      // Fritz! Fehler
      fErrorOccured:= True;
      fErrorNr:= PResult;
      fBusy:= False;
      Result:= False;
      case PResult of
        FDERR_GENERALFAILURE: fErrorMessage:= FDERRS_GENERALFAILURE;
        FDERR_FILETRANSFER: fErrorMessage:= FDERRS_FILETRANSFER;
        FDERR_FILEACCESS: fErrorMessage:= FDERRS_FILEACCESS;
        FDERR_PATHNOTFOUND: fErrorMessage:= FDERRS_PATHNOTFOUND;
        FDERR_ACCESSDENIED: fErrorMessage:= FDERRS_ACCESSDENIED;
        FDERR_FILEOPEN: fErrorMessage:= FDERRS_FILEOPEN;
        FDERR_TIMEOUT: fErrorMessage:= FDERRS_TIMEOUT;
        FDERR_LOGINFAILURE: fErrorMessage:= FDERRS_LOGINFAILURE;
        FDERR_FILEALREADYEXISTS: fErrorMessage:= FDERRS_FILEALREADYEXISTS;
        FDERR_IDLETIMEOUT: fErrorMessage:= FDERRS_IDLETIMEOUT;
        FDERR_DOSERROR: fErrorMessage:= FDERRS_DOSERROR;
        FDERR_NOREPLY: fErrorMessage:= FDERRS_NOREPLY;
        FDERR_NODISKSPACE: fErrorMessage:= FDERRS_NODISKSPACE;
        FDERR_DISKERROR: fErrorMessage:= FDERRS_DISKERROR;
        FDERR_DATAEXPECTED: fErrorMessage:= FDERRS_DATAEXPECTED;
        FDERR_NOACK: fErrorMessage:= FDERRS_NOACK;
        FDERR_INVALIDRECIPIENT: fErrorMessage:= FDERRS_INVALIDRECIPIENT;
        FDERR_OUTOFMEMORY: fErrorMessage:= FDERRS_OUTOFMEMORY;
        FDERR_PATHACCESS: fErrorMessage:= FDERRS_PATHACCESS;
        FDERR_FILENOTFOUND: fErrorMessage:= FDERRS_FILENOTFOUND;
        FDERR_CONNCOLLISION: fErrorMessage:= FDERRS_CONNCOLLISION;
        FDERR_ABORT: fErrorMessage:= FDERRS_ABORT;
        FDERR_INCOMPRECIPIENT: fErrorMessage:= FDERRS_INCOMPRECIPIENT;
        else fErrorMessage:= FDERRS_GENERALFAILURE;
      end;
    end;
    5:
    begin
      // Fritz!Data beschftigt
      fErrorOccured:= True;
      fErrorNr:= FDERR_NOTREADY;
      fErrorMessage:= FDERRS_NOTREADY;
      Result:= False;
    end
    else
    begin
      // unbekannter Fehler
      fErrorOccured:= True;
      fErrorNr:= FDERR_UNKNOWN;
      fErrorMessage:= FDERRS_UNKNOWN;
      fBusy:= False;
      Result:= False;
    end;
  end;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.Connect
//  Stellt eine Verbindung zu einem Server her.
//  Adress = Telefonnummer
//  Username = Benutzername
//  Password = Passwort
//  Options = Welche Art von Verbindung ?
//----------------------------------------------------------------------}
procedure TFritzDataControl.Connect(Address, UserName, Password: string; Options:
  TFritzDataConnectOptions);
var
  PResult: Integer;
  Protocol: SmallInt;
begin
  ClearErrorState;
  if Busy then
  begin
    fErrorOccured:= True;
    fErrorNr:= FDERR_NOTREADY;
    fErrorMessage:= FDERRS_NOTREADY;
    Exit;
  end;

  // Options auswerten
  Protocol:= 0;
  if fdcIDTrans in Options then
  begin
    Protocol:= Protocol or 256;
    if fdcTwoChannel in Options then
    begin
      Protocol:= Protocol or 1024;
    end;
    if fdcIDTransCompression in Options then
    begin
      Protocol:= Protocol or 2048;
    end;
  end
  else
  begin
    Protocol:= Protocol or 512; // EuroFile
  end;

  if fdcV42Compression in Options then
  begin
    Protocol:= Protocol or 4096;
  end;

  // Verbindung herstellen
  fBusy:= True;
  fFritzData.Connect(Address, UserName, Password, Protocol, PResult);
  HandlePResult(PResult);
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.Disconnect
//  Beendet die Verbindung zu einem Server
//----------------------------------------------------------------------}
procedure TFritzDataControl.Disconnect;
var
  PResult: Integer;
begin
  ClearErrorState;
  if not fConnected then
  begin
    fErrorOccured:= True;
    fErrorNr:= FDERR_NOTPOSSIBLE;
    fErrorMessage:= FDERRS_NOTPOSSIBLE;
    Exit;
  end;

  fBusy:= True;
  fFritzData.Disconnect(PResult);
  HandlePResult(PResult);
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.Abort
//  Bricht aktuell laufende Aktionen, i.A. Dateitransfer ab.
//----------------------------------------------------------------------}
procedure TFritzDataControl.Abort;
var
  PResult: Integer;
begin
  ClearErrorState;
  if not fBusy then
  begin
    fErrorOccured:= True;
    fErrorNr:= FDERR_NOTPOSSIBLE;
    fErrorMessage:= FDERRS_NOTPOSSIBLE;
    Exit;
  end;

  fBusy:= True;
  fFritzData.Abort(PResult);
  HandlePResult(PResult);
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.DirRemote
//  Ermittelt eine Liste aller Dateien/Verzeichnisse im aktuellen
//  Verzeichnis auf dem Server
//----------------------------------------------------------------------}
procedure TFritzDataControl.DirRemote;
var
  PResult: Integer;
  Select: SmallInt;
begin
  fFiles.Clear;
  fDirectories.Clear;
  ClearErrorState;

  if not CheckReady then
    Exit;

  Select:= 2 + 4; // entferntes Verzeichnis + neu einlesen
  fBusy:= True;
  fFritzData.Directory(Select, PResult);
  HandlePResult(PResult);
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.DirLocal
//  Ermittelt eine Liste aller Dateien/Verzeichnisse im aktuellen
//  lokalen Verzeichnis
//----------------------------------------------------------------------}
procedure TFritzDataControl.DirLocal;
var
  PResult: Integer;
  Select: SmallInt;
begin
  fFiles.Clear;
  fDirectories.Clear;
  ClearErrorState;

  if not CheckLocalReady then
    Exit;

  Select:= 1 + 4; // lokales Verzeichnis + neu einlesen
  fBusy:= True;
  fFritzData.Directory(Select, PResult);
  HandlePResult(PResult);
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.GetNextDirItem
//  Dient zum Ermitteln der gelesenen Dateien/Verzeichnisse
//  Rckgabewert gibt an, ob Datei, Verzeichnis oder Ende der Liste
//  Daten werden in FileInfo zurckgegeben
//----------------------------------------------------------------------}
function TFritzDataControl.GetNextDirItem(var FileInfo: TFritzDataFileInfo):
  TFritzDataFileType;
var
  InfoString: string;
  Name, Size, Date, Time: string;

  {/----------------------------------------------------------------------
  //  Leere Daten zurckgeben
  //----------------------------------------------------------------------}
  procedure ClearFileInfo;
  begin
    FileInfo.FileName:= EmptyStr;
    FileInfo.Size:= 0;
    FileInfo.LastChange:= 0;
  end;

  {/----------------------------------------------------------------------
  //  Alle Punkte aus einem String lschen
  //----------------------------------------------------------------------}
  procedure ClearDots(var S: String);
  var
    P: Integer;
  begin
    repeat
      P:= Pos('.', S);
      if P > 0 then
        Delete(S, P, 1);
    until P = 0;
  end;
  
  {/----------------------------------------------------------------------
  //  Von Fritz!Data bergebenen String in seine Bestandteile zerlegen
  //----------------------------------------------------------------------}
  procedure ParseInfoString;
  var
    P: Integer;
  begin
    P:= Pos(#9, InfoString);
    Name:= Copy(InfoString, 1, P - 1);
    Delete(InfoString, 1, P);
    P:= Pos(#9, InfoString);
    Size:= Copy(InfoString, 1, P - 1);
    ClearDots(Size);
    Delete(InfoString, 1, P);
    P:= Pos(#9, InfoString);
    Date:= Copy(InfoString, 1, P - 1);
    Delete(InfoString, 1, P);
    Time:= InfoString;
  end;

  {/----------------------------------------------------------------------
  //  Aus Datum und Uhrzeit als String ein TDateTime zusammensetzen
  //----------------------------------------------------------------------}
  function ComposeLastChange: TDateTime;
  begin
    Result:= 0;
    try
      Result:= Result + StrToDate(Date);
    except
    end;
    try
      Result:= Result + StrToTime(Time);
    except
    end;
  end;

begin
  if fDirectories.Count = 0 then
  begin
    if fFiles.Count = 0 then
    begin
      // Keine Eintrge mehr vorhanden
      Result:= fdftEOL;
      ClearFileInfo;
    end
    else
    begin
      // Datei gefunden
      Result:= fdftFile;

      InfoString:= fFiles[0];
      fFiles.Delete(0);

      ParseInfoString;
      FileInfo.Typ:= fdftFile;
      FileInfo.FileName:= Name;
      FileInfo.Size:= StrToIntDef(Size, 0);
      FileInfo.LastChange:= ComposeLastChange;
    end;
  end
  else
  begin
    // Verzeichnis gefunden
    Result:= fdftDirectory;

    InfoString:= fDirectories[0];
    fDirectories.Delete(0);

    ParseInfoString;
    FileInfo.Typ:= fdftDirectory;
    FileInfo.FileName:= Name;
    FileInfo.Size:= 0;
    FileInfo.LastChange:= ComposeLastChange;
  end;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.ChDirLocal
//  Aktuelles lokales Verzeichnis/Laufwerk wechseln
//----------------------------------------------------------------------}
procedure TFritzDataControl.ChDirLocal(NewPath: string);
var
  PResult: Integer;
  Directory: WideString;
  Select: SmallInt;
begin
  ClearErrorState;

  if not CheckLocalReady then
    Exit;

  Select:= 1; // lokales Verzeichnis
  Directory:= NewPath;
  fBusy:= True;
  fFritzData.ChangeDirectory(Select, Directory, PResult);
  HandlePResult(PResult);
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.ChDirRemote
//  Aktuelles Verzeichnis/Laufwerk auf dem Server wechseln
//----------------------------------------------------------------------}
procedure TFritzDataControl.ChDirRemote(NewPath: string);
var
  PResult: Integer;
  Directory: WideString;
  Select: SmallInt;
begin
  ClearErrorState;

  if not CheckReady then
    Exit;

  Select:= 2; // Verzeichnis auf Server
  Directory:= NewPath;
  fBusy:= True;
  fFritzData.ChangeDirectory(Select, Directory, PResult);
  HandlePResult(PResult);
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.GetCWDLocal
//  Ermittelt das aktuelle lokale Verzeichnis
//----------------------------------------------------------------------}
function TFritzDataControl.GetCWDLocal: string;
var
  PInfo: OleVariant;
  PResult: Integer;
  Flag: SmallInt;
begin
  Result:= EmptyStr;
  ClearErrorState;

  if not CheckLocalReady then
    Exit;

  Flag:= 1 + 256 + 512; // lokales Verzeichnis + aktuelles Verzeichnis ermitteln
  PInfo:= Unassigned;
  fFritzData.GetInfo(Flag, PInfo, PResult);
  if HandlePResult(PResult) then
  begin
    Result:= PInfo;
  end;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.GetCWDRemote
//  Ermittelt das aktuelle Verzeichnis auf dem Server
//----------------------------------------------------------------------}
function TFritzDataControl.GetCWDRemote: string;
var
  PInfo: OleVariant;
  PResult: Integer;
  Flag: SmallInt;
begin
  Result:= EmptyStr;
  ClearErrorState;

  if not CheckReady then
    Exit;

  Flag:= 2 + 256 + 512; // entferntes Verzeichnis + aktuelles Verzeichnis ermitteln
  PInfo:= Unassigned;
  fFritzData.GetInfo(Flag, PInfo, PResult);
  if HandlePResult(PResult) then
  begin
    Result:= PInfo;
  end;
end;

{/----------------------------------------------------------------------
//  Funktion DWordToDriveList
//  Gibt anhand eines von Fritz!data gelieferten 32bit-Wertes einen
//  String mit allen verfgbaren Laufwerken zurck;
//----------------------------------------------------------------------}
function DWordToDriveList(Inp: DWord): String;
var
  CH: Char;
begin
  Result:= EmptyStr;
  for CH:= 'A' to 'Z' do
  begin
    if (Inp and 1) <> 0 then
      Result:= Result + CH;
    Inp:= Inp shr 1;
  end;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.GetDriveListLocal
//  Ermittelt lokal verfgbare Laufwerke.
//----------------------------------------------------------------------}
function TFritzDataControl.GetDriveListLocal: string;
var
  PInfo: OleVariant;
  PResult: Integer;
  Flag: SmallInt;
begin
  Result:= EmptyStr;
  ClearErrorState;

  if not CheckLocalReady then
    Exit;

  Flag:= 1 + 256 + 1024; // lokales Verzeichnis + Laufwerke ermitteln
  PInfo:= Unassigned;
  fFritzData.GetInfo(Flag, PInfo, PResult);
  if HandlePResult(PResult) then
  begin
    // Ergebnis korrekt interpretieren und String mit
    // Laufwerken zurckgeben
    Result:= DWordToDriveList(PInfo);
  end;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.GetDriveListRemote
//  Ermittelt auf dem Server verfgbare Laufwerke.
//----------------------------------------------------------------------}
function TFritzDataControl.GetDriveListRemote: string;
var
  PInfo: OleVariant;
  PResult: Integer;
  Flag: SmallInt;
begin
  Result:= EmptyStr;
  ClearErrorState;

  if not CheckReady then
    Exit;

  Flag:= 2 + 256 + 1024; // entferntes Verzeichnis + Laufwerke ermitteln
  PInfo:= Unassigned;
  fFritzData.GetInfo(Flag, PInfo, PResult);
  if HandlePResult(PResult) then
  begin
    // Ergebnis korrekt interpretieren und String mit
    // Laufwerken zurckgeben
    Result:= DWordToDriveList(PInfo);
  end;
end;


{/----------------------------------------------------------------------
//  Methode TFritzDataControl.MkDirLocal
//  Erstellt lokal ein neues Verzeichnis im aktuellen Verzeichnis
//----------------------------------------------------------------------}
procedure TFritzDataControl.MkDirLocal(DirName: string);
var
  Select: SmallInt;
  Directory: WideString;
  PResult: Integer;
begin
  ClearErrorState;

  if not CheckLocalReady then
    Exit;

  Select:= 1; // lokales Verzeichnis
  Directory:= DirName;
  fBusy:= True;
  fFritzData.MakeDirectory(Select, Directory, PResult);
  HandlePResult(PResult);
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.MkDirRemote
//  Erstellt auf dem Server ein neues Verzeichnis im aktuellen
//  Verzeichnis
//----------------------------------------------------------------------}
procedure TFritzDataControl.MkDirRemote(DirName: string);
var
  Select: SmallInt;
  Directory: WideString;
  PResult: Integer;
begin
  ClearErrorState;

  if not CheckReady then
    Exit;

  Select:= 2; // entferntes Verzeichnis
  Directory:= DirName;
  fBusy:= True;
  fFritzData.MakeDirectory(Select, Directory, PResult);
  HandlePResult(PResult);
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.DeleteLocal
//  Lscht eine oder mehrere Dateien oder Verzeichnisse aus dem aktuellen
//  lokalen Verzeichnis
//  Filename kann auch die Wildcards * und ? enthalten
//  Verzeichnisse werden inklusive enthaltenen Dateien und
//  Unterverzeichnisse gelscht, wenn mglich
//----------------------------------------------------------------------}
procedure TFritzDataControl.DeleteLocal(FileName: string);
var
  Select: SmallInt;
  Name: WideString;
  PResult: Integer;
begin
  ClearErrorState;

  if not CheckLocalReady then
    Exit;

  Select:= 1; // lokales Verzeichnis
  Name:= FileName;
  fBusy:= True;
  fFritzData.Delete(Select, Name, PResult);
  HandlePResult(PResult);
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.DeleteRemote
//  Lscht eine oder mehrere Dateien oder Verzeichnisse aus dem aktuellen
//  Verzeichnis auf dem Server
//  Filename kann auch die Wildcards * und ? enthalten
//  Verzeichnisse werden inklusive enthaltenen Dateien und
//  Unterverzeichnisse gelscht, wenn mglich
//----------------------------------------------------------------------}
procedure TFritzDataControl.DeleteRemote(FileName: string);
var
  Select: SmallInt;
  Name: WideString;
  PResult: Integer;
begin
  ClearErrorState;

  if not CheckReady then
    Exit;

  Select:= 2; // entferntes Verzeichnis
  Name:= FileName;
  fBusy:= True;
  fFritzData.Delete(Select, Name, PResult);
  HandlePResult(PResult);
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.Upload
//  Kopiert eine oder mehrere Dateien oder Verzeichnisse aus dem lokalen
//  Verzeichnis in das Verzeichnis auf dem Server
//  Filename kann auch die Wildcards * und ? enthalten
//  Verzeichnisse werden inklusive enthaltenen Dateien und
//  Unterverzeichnisse mitkopiert, wenn mglich
//  Ist Move True, werden die Dateien im lokalen Verzeichnis
//  anschlieend gelscht.
//----------------------------------------------------------------------}
procedure TFritzDataControl.Upload(FileName: String; Move: Boolean);
var
  Select: SmallInt;
  Name: WideString;
  PResult: Integer;
begin
  ClearErrorState;

  if not CheckReady then
    Exit;

  Select:= 1 + 4; // vom lokalen ins entfernte Verzeichnis, alle berschreiben
  Name:= FileName;
  fBusy:= True;
  if Move then
  begin
    fFritzData.Move(Select, Name, PResult);
  end
  else
  begin
    fFritzData.Copy(Select, Name, PResult);
  end;
  HandlePResult(PResult);
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.Download
//  Kopiert eine oder mehrere Dateien oder Verzeichnisse aus dem
//  Verzeichnis auf dem Server ins lokale Verzeichnis
//  Filename kann auch die Wildcards * und ? enthalten
//  Verzeichnisse werden inklusive enthaltenen Dateien und
//  Unterverzeichnisse mitkopiert, wenn mglich
//  Ist Move True, werden die Dateien im entfernten Verzeichnis
//  anschlieend gelscht.
//----------------------------------------------------------------------}
procedure TFritzDataControl.Download(FileName: String; Move: Boolean);
var
  Select: SmallInt;
  Name: WideString;
  PResult: Integer;
begin
  ClearErrorState;

  if not CheckReady then
    Exit;

  Select:= 2 + 4; // vom entfernten ins lokale Verzeichnis, alle berschreiben
  Name:= FileName;
  fBusy:= True;
  if Move then
  begin
    fFritzData.Move(Select, Name, PResult);
  end
  else
  begin
    fFritzData.Copy(Select, Name, PResult);
  end;
  HandlePResult(PResult);
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.GetServerName
//  Ermittelt Namen des verbundenen Servers
//----------------------------------------------------------------------}
function TFritzDataControl.GetServerName: string;
var
  PInfo: OleVariant;
  PResult: Integer;
  Flag: SmallInt;
begin
  Result:= EmptyStr;
  ClearErrorState;

  if not CheckReady then
    Exit;

  Flag:= 512; // Get ServerName
  PInfo:= Unassigned;
  fFritzData.GetInfo(Flag, PInfo, PResult);
  if HandlePResult(PResult) then
  begin
    Result:= PInfo;
  end;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.LFNSupported
//  Gibt an, ob der Server lange Dateinamen untersttzt oder nicht
//----------------------------------------------------------------------}
function TFritzDataControl.LFNSupported: Boolean;
var
  PInfo: OleVariant;
  PResult: Integer;
  Flag: SmallInt;
begin
  Result:= False;
  ClearErrorState;

  if not CheckReady then
    Exit;

  Flag:= 1024; // Get LFNSupported
  PInfo:= Unassigned;
  fFritzData.GetInfo(Flag, PInfo, PResult);
  if HandlePResult(PResult) then
  begin
    Result:= PInfo;
  end;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.GetConnStatus
//  Ermittelt Eigenschaften der aktuellen Verbindung
//----------------------------------------------------------------------}
function TFritzDataControl.GetConnStatus: TFritzDataConnectOptions;
var
  PInfo: OleVariant;
  PResult: Integer;
  Flag: SmallInt;
  Info: Word;
begin
  Result:= [];
  ClearErrorState;

  if not CheckReady then
    Exit;

  Flag:= 256; // Get ConnInfo
  PInfo:= Unassigned;
  fFritzData.GetInfo(Flag, PInfo, PResult);
  if HandlePResult(PResult) then
  begin
    Info:= PInfo;

    if (Info and 256) <> 0 then
    begin
      Result:= Result + [fdcIDTrans];
      if (Info and 1024) <> 0 then
      begin
        Result:= Result + [fdcTwoChannel];
      end;
      if (Info and 2048) <> 0 then
      begin
        Result:= Result + [fdcIDTransCompression];
      end;
    end;
    if (Info and 4096) <> 0 then
    begin
      Result:= Result + [fdcV42Compression];
    end;
  end;
end;

{/----------------------------------------------------------------------
//  Methode TFritzDataControl.GetCAPIISDNErrorMessage
//  Ermittelt Fehlertext zu angegebenem CAPI- oder ISDN-Fehler
//----------------------------------------------------------------------}
function TFritzDataControl.GetCAPIISDNErrorMessage(Error: Word): String;
var
  PInfo: OleVariant;
  PResult: Integer;
  Flag: SmallInt;
begin
  Result:= EmptyStr;
  ClearErrorState;

  if not CheckLocalReady then
    Exit;

  Flag:= 512 + 1024; // Get Error Message
  PInfo:= SmallInt(Error);
  VarCast(PInfo, PInfo, varSmallint);
  fFritzData.GetInfo(Flag, PInfo, PResult);
  if HandlePResult(PResult) then
  begin
    Result:= PInfo;
  end;
end;

initialization
end.
