unit FritzDataControlAuto;

interface

uses
  ComObj, ActiveX, FritzDataControlLib_TLB, StdVcl, FritzDataControl;

type
  TFritzDataControlDocument = class(TAutoObject, IFritzDataControlDocument)
  private
    FritzDataControl: TFritzDataControl;
    procedure CheckConnected;
  protected
    function FritzDataAlreadyRuns: WordBool; safecall;
    procedure FritzDataConnect; safecall;
    procedure FritzDataDisconnect; safecall;
    function Get_CWDLocal: WideString; safecall;
    function Get_Busy: WordBool; safecall;
    function Get_LocalReady: WordBool; safecall;
    function Get_Ready: WordBool; safecall;
    procedure DirLocal; safecall;
    function GetNextDirItem(
      out FileInfo: FritzDataFileInfo): FritzDataFileType; safecall;
    procedure Connect(const Address, UserName, Password: WideString;
      Options: SYSINT); safecall;
    procedure ChDirLocal(const NewPath: WideString); safecall;
    procedure DirRemote; safecall;
    procedure Disconnect; safecall;
    procedure ChDirRemote(const NewPath: WideString); safecall;
    procedure MkDirLocal(const DirName: WideString); safecall;
    procedure MkDirRemote(const DirName: WideString); safecall;
    procedure DeleteLocal(const FileName: WideString); safecall;
    procedure DeleteRemote(const FileName: WideString); safecall;
    procedure Upload(const FileName: WideString; Move: WordBool); safecall;
    procedure Download(const FileName: WideString; Move: WordBool); safecall;
    procedure Abort; safecall;
    function Get_CWDRemote: WideString; safecall;
    function Get_DriveListLocal: WideString; safecall;
    function Get_DriveListRemote: WideString; safecall;
    function Get_ServerName: WideString; safecall;
    function Get_LFNSupported: WordBool; safecall;
    function Get_ConnStatus: SYSINT; safecall;
    function GetCAPIISDNErrorMessage(Error: SYSINT): WideString; safecall;
    function Get_ErrorOccured: WordBool; safecall;
    function Get_ErrorNr: SYSINT; safecall;
    function Get_ErrorMessage: WideString; safecall;
  public
    procedure Initialize; override;
    destructor Destroy; override;
  end;

implementation

uses SysUtils, ComServ;

procedure TFritzDataControlDocument.Initialize;
begin
  inherited;
  FritzDataControl:= nil;
end;

destructor TFritzDataControlDocument.Destroy;
begin
  FritzDataDisconnect;
  inherited;
end;

function TFritzDataControlDocument.FritzDataAlreadyRuns: WordBool;
begin
  Result:= TFritzDataControl.FritzDataAlreadyRuns;
end;

procedure TFritzDataControlDocument.FritzDataConnect;
begin
  if not Assigned(FritzDataControl) then
   FritzDataControl:= TFritzDataControl.Create;
end;

procedure TFritzDataControlDocument.FritzDataDisconnect;
begin
  if Assigned(FritzDataControl) then
  begin
    FritzDataControl.Free;
    FritzDataControl:= nil;
  end;
end;

function TFritzDataControlDocument.Get_CWDLocal: WideString;
begin
  CheckConnected;

  Result:= FritzDataControl.GetCWDLocal;
end;

procedure TFritzDataControlDocument.CheckConnected;
begin
  if not Assigned(FritzDataControl) then
    raise Exception.Create('Call FritzDataConnect first');
end;

function TFritzDataControlDocument.Get_Busy: WordBool;
begin
  CheckConnected;

  Result:= FritzDataControl.Busy;
end;

function TFritzDataControlDocument.Get_LocalReady: WordBool;
begin
  CheckConnected;

  Result:= FritzDataControl.LocalReady;
end;

function TFritzDataControlDocument.Get_Ready: WordBool;
begin
  CheckConnected;

  Result:= FritzDataControl.Ready;
end;

procedure TFritzDataControlDocument.DirLocal;
begin
  CheckConnected;

  FritzDataControl.DirLocal;
end;

function TFritzDataControlDocument.GetNextDirItem(
  out FileInfo: FritzDataFileInfo): FritzDataFileType;
var
  Info: TFritzDataFileInfo;
  Res: TFritzDataFileType;
begin
  CheckConnected;

  Res:= FritzDataControl.GetNextDirItem(Info);

  case Res of
    fdftDirectory: Result:= FDFT_DIRECTORY;
    fdftFile: Result:= FDFT_FILE;
    fdftEOL: Result:= FDFT_EOL;
  end;

  FileInfo.Typ:= Result;
  FileInfo.FileName:= Info.FileName;
  FileInfo.Size:= Info.Size;
  FileInfo.LastChange:= Info.LastChange;
end;

procedure TFritzDataControlDocument.Connect(const Address, UserName,
  Password: WideString; Options: SYSINT);
var
  Op: TFritzDataConnectOptions;
begin
  CheckConnected;

  Op:= [];
  if (Options and FDCO_IDTRANS) > 0 then
    Include(Op, fdcIDTrans);
  if (Options and FDCO_TWOCHANNEL) > 0 then
    Include(Op, fdcTwoChannel);
  if (Options and FDCO_IDTRANSCOMPRESSION) > 0 then
    Include(Op, fdcIDTransCompression);
  if (Options and FDCO_V42COMPRESSION) > 0 then
    Include(Op, fdcV42Compression);

  FritzDataControl.Connect(Address, UserName, Password, Op);
end;

procedure TFritzDataControlDocument.ChDirLocal(const NewPath: WideString);
begin
  CheckConnected;

  FritzDataControl.ChDirLocal(NewPath);
end;

procedure TFritzDataControlDocument.DirRemote;
begin
  CheckConnected;

  FritzDataControl.DirRemote;
end;

procedure TFritzDataControlDocument.Disconnect;
begin
  CheckConnected;

  FritzDataControl.Disconnect;
end;

procedure TFritzDataControlDocument.ChDirRemote(const NewPath: WideString);
begin
  CheckConnected;

  FritzDataControl.ChDirRemote(NewPath);
end;

procedure TFritzDataControlDocument.MkDirLocal(const DirName: WideString);
begin
  CheckConnected;

  FritzDataControl.MkDirLocal(DirName);
end;

procedure TFritzDataControlDocument.MkDirRemote(const DirName: WideString);
begin
  CheckConnected;

  FritzDataControl.MkDirRemote(DirName);
end;

procedure TFritzDataControlDocument.DeleteLocal(const FileName: WideString);
begin
  CheckConnected;

  FritzDataControl.DeleteLocal(FileName);
end;

procedure TFritzDataControlDocument.DeleteRemote(const FileName: WideString);
begin
  CheckConnected;

  FritzDataControl.DeleteRemote(FileName);
end;

procedure TFritzDataControlDocument.Upload(const FileName: WideString;
  Move: WordBool);
begin
  CheckConnected;

  FritzDataControl.Upload(FileName, Move);
end;

procedure TFritzDataControlDocument.Download(const FileName: WideString;
  Move: WordBool);
begin
  CheckConnected;

  FritzDataControl.Download(FileName, Move);
end;

procedure TFritzDataControlDocument.Abort;
begin
  CheckConnected;

  FritzDataControl.Abort;
end;

function TFritzDataControlDocument.Get_CWDRemote: WideString;
begin
  CheckConnected;

  Result:= FritzDataControl.GetCWDRemote;
end;

function TFritzDataControlDocument.Get_DriveListLocal: WideString;
begin
  CheckConnected;

  Result:= FritzDataControl.GetDriveListLocal;
end;

function TFritzDataControlDocument.Get_DriveListRemote: WideString;
begin
  CheckConnected;

  Result:= FritzDataControl.GetDriveListRemote
end;

function TFritzDataControlDocument.Get_ServerName: WideString;
begin
  CheckConnected;

  Result:= FritzDataControl.GetServerName;
end;

function TFritzDataControlDocument.Get_LFNSupported: WordBool;
begin
  CheckConnected;

  Result:= FritzDataControl.LFNSupported;
end;

function TFritzDataControlDocument.Get_ConnStatus: SYSINT;
var
  Op: TFritzDataConnectOptions;
begin
  CheckConnected;

  Result:= 0;

  Op:= FritzDataControl.GetConnStatus;

  if fdcIDTrans in Op then
    Result:= Result + FDCO_IDTRANS;
  if fdcTwoChannel in Op then
    Result:= Result + FDCO_TWOCHANNEL;
  if fdcIDTransCompression in Op then
    Result:= Result + FDCO_IDTRANSCOMPRESSION;
  if fdcV42Compression in Op then
    Result:= Result + FDCO_V42COMPRESSION;
end;

function TFritzDataControlDocument.GetCAPIISDNErrorMessage(
  Error: SYSINT): WideString;
begin
  CheckConnected;

  Result:= FritzDataControl.GetCAPIISDNErrorMessage(Error)
end;

function TFritzDataControlDocument.Get_ErrorOccured: WordBool;
begin
  CheckConnected;

  Result:= FritzDataControl.ErrorOccured;
end;

function TFritzDataControlDocument.Get_ErrorNr: SYSINT;
begin
  CheckConnected;

  Result:= FritzDataControl.ErrorNr;
end;

function TFritzDataControlDocument.Get_ErrorMessage: WideString;
begin
  CheckConnected;

  Result:= FritzDataControl.ErrorMessage;
end;

initialization
  TAutoObjectFactory.Create(ComServer, TFritzDataControlDocument, Class_FritzDataControlDocument,
    ciMultiInstance, tmApartment);
end.
